local SOURCE_NAV1 = "NAV1"
local SOURCE_NAV2 = "NAV2"
local SOURCE_GPS = "GPS"
local ac_heading = 0
local bearings = nil
local data_sources = nil
local vis_index = 0
local vis_mask = 3
local avionics_power = false
local views = nil
local selected_view = 360
local source_colours = nil

local function update_data_source(source, bv, b, dv, d)
    data_sources[source] = {
        bearing_valid = bv,
        bearing = b,
        dist_valid = dv,
        dist = d
    }
end

local function draw_head1(view, colour)
    centre = view.centre
    radius = view.size + 3

    for i = centre.y - radius, centre.y - radius + 8 do
        _move_to(centre.x, i)
        _line_to(centre.x - 30, i + 20)
        _move_to(centre.x, i)
        _line_to(centre.x + 30, i + 20)
        _stroke(colour, 1.5)
    end
end

local function draw_tail1(view, colour)
    centre = view.centre
    radius = view.size + 3

    _move_to(centre.x, centre.y + radius)
    _line_to(centre.x, centre.y + radius - 25)
    _stroke(colour, 6)
end

local function draw_head2(view, colour)
    centre = view.centre
    radius = view.size - 9

    for i = centre.y - radius, centre.y - radius + 8 do
        _move_to(centre.x - 3, i)
        _line_to(centre.x - 30, i + 18)
        _move_to(centre.x + 3, i)
        _line_to(centre.x + 30, i + 18)
        _stroke(colour, 1.5)
    end
end

local function draw_tail2(view, colour)
    centre = view.centre
    radius = view.size - 9

    _move_to(centre.x - 6, centre.y + radius)
    _line_to(centre.x - 6, centre.y + radius - 25)
    _move_to(centre.x + 6, centre.y + radius)
    _line_to(centre.x + 6, centre.y + radius - 25)
    _stroke(colour, 6)
end

local BearingPtr = {
    set_source = function (self, src)
        self.source = src
    end,

    draw = function (self)
        source = data_sources[self.source]
        self.pointer:draw(0, source.bearing, views[selected_view], source_colours[self.source])
    end,

    visible = function (self, visible)
        source = data_sources[self.source]
        ptr_visible = visible and source.bearing_valid

        self.pointer:visible(ptr_visible)

        lower_display_bearing_visible(self.index, visible)
        upper_display_park_bearing_ptr(self.index, ptr_visible and helpers_park_pointer(0, source.bearing, selected_view))
    end


}

function BearingPtr:create(index, src, canvas_size, draw_head, draw_tail)
    result =  {
        index = index,
        source = src,
        pointer = Pointer:create(canvas_size, draw_head, draw_tail)
    }

    setmetatable(result, self)
    self.__index = self

    return result
end

function bearings_init(centre, canvas_size)
    views = {}
    views[360] = View.create(centre, 176, 360)
    views[70] = View.create({x = centre.x, y = 400}, 326, 70)

    source_colours = {}
    source_colours[SOURCE_NAV1] = "green"
    source_colours[SOURCE_NAV2] = "yellow"
    source_colours[SOURCE_GPS] = "cyan"

    selected_view = 360

    data_sources = {}
    update_data_source(SOURCE_NAV1, false, 0, false, 0)
    update_data_source(SOURCE_NAV2, false, 0, false, 0)
    update_data_source(SOURCE_GPS, false, 0, false, 0)

    bearings = {
        BearingPtr:create(1, SOURCE_NAV1, canvas_size, draw_head1, draw_tail1),
        BearingPtr:create(2, SOURCE_NAV2, canvas_size, draw_head2, draw_tail2)
    }

    helpers_visit_array(bearings, function (index, bearing)
        bearing:visible(false)
    end)

end

function bearings_update_source_data(heading,
                                    has_nav1, rel_bearing1, has_dme1, dme_dist1,
                                    has_nav2, rel_bearing2, has_dme2, dme_dist2)
    ac_heading = heading
    update_data_source(SOURCE_NAV1, has_nav1, rel_bearing1, has_dme1, dme_dist1)
    update_data_source(SOURCE_NAV2, has_nav2, rel_bearing2, has_dme2, dme_dist2)

    helpers_visit_array(bearings, function (index, bearing)
        source = data_sources[bearing.source]
        lower_display_update_bearing_data(index, bearing.source,
                                          source.dist, source.dist_valid,
                                          source.bearing, source.bearing_valid)
        bearing:draw()
    end)
end

function bearings_select()
    vis_index = (vis_index + 1) & vis_mask

    helpers_visit_array(bearings, function (index, bearing)
        bearing:visible(((vis_index & index) ~= 0) and avionics_power)
      end)
end

function bearings_power_on(val)
    avionics_power = val
    helpers_visit_array(bearings, function (index, bearing)
        bearing:visible(((vis_index & index) ~= 0) and avionics_power)
    end)
end

function bearings_set_view(view)
    selected_view = view

    helpers_visit_array(bearings, function (index, bearing)
        bearing:draw()
    end)
end
