local rose_txt_style = "font:inconsolata_regular.ttf; size:24; valign:center; halign:center; color:white;"
local heading_txt_style = "font:inconsolata_bold.ttf; size:30; valign:center; halign:center; color:white;"

local heading = 0
local heading_bug_setting = 0

local enabled_compass_view = 1
local compass_views = nil
local compass = nil

local function draw_heading_bug(view)
    centre = view.centre
    radius = view.size
    height = 20
    width = 60
    top = centre.y - radius - height
    bottom = top + height
    rect_width = 18
    triangle_width = (width / 2) - rect_width
    _rect(centre.x - (width / 2), top, rect_width, height)
    _fill("white")
    _rect(centre.x + triangle_width, top, rect_width, height)
    _fill("white")
    _triangle(centre.x - triangle_width, top, centre.x - triangle_width, bottom, centre.x, bottom)
    _fill("white")
    _triangle(centre.x + triangle_width, top, centre.x + triangle_width, bottom, centre.x, bottom)
    _fill("white")
end

local Rose = {
    draw = function (self)
        view = compass_views[enabled_compass_view]
        radius = view.size
        centre = view.centre
        line_lengths = {10, 15, 10, 15, 10}
        line_lengths[0] = 20

        canvas_draw(self.canvas,
            function()
                for i = 0, 360, 5 do
                    rotation = i - heading
                    top = centre.y - radius
                    _rotate(rotation, centre.x, centre.y)
                    _move_to(centre.x, top)
                    _line_to(centre.x, top + (line_lengths[(i / 5) % 6]))
                    _stroke("white", 2)
                    _rotate(-rotation, centre.x, centre.y)
                end

                bottom = centre.y - radius
                top = bottom - 25
                width = 14
                _triangle(centre.x, bottom, centre.x - width, top, centre.x + width, top)
                _fill("white")
            end)

        for i = 1, 12 do
        	deg = heading + 30 * (i - 1)
            helpers_rotate_around_pt(self.txt[i], centre, radius - self.txt_radius_delta, deg, 20, 20, true)
        end

        if view.viewable < 90 then
            offset = math.sin(math.rad(view.viewable / 2)) * radius
            viewport_rect(self.group, centre.x - offset, 0, 2 * offset, self.canvas_size)
        else
            viewport_rect(self.group, 0, 0, self.canvas_size, self.canvas_size)
        end
    end,

    visible = function (self, val)
        visible(self.group, val)
    end
}

function Rose:create(canvas_size)
    rose_canvas = canvas_add(0, 0, canvas_size, canvas_size)
    rose_txt = {
        txt_add("N", rose_txt_style, 0, 0, canvas_size, canvas_size),
        txt_add("33", rose_txt_style, 0, 0, canvas_size, canvas_size),
        txt_add("30", rose_txt_style, 0, 0, canvas_size, canvas_size),
        txt_add("W", rose_txt_style, 0, 0, canvas_size, canvas_size),
        txt_add("24", rose_txt_style, 0, 0, canvas_size, canvas_size),
        txt_add("21", rose_txt_style, 0, 0, canvas_size, canvas_size),
        txt_add("S", rose_txt_style, 0, 0, canvas_size, canvas_size),
        txt_add("15", rose_txt_style, 0, 0, canvas_size, canvas_size),
        txt_add("12", rose_txt_style, 0, 0, canvas_size, canvas_size),
        txt_add("E", rose_txt_style, 0, 0, canvas_size, canvas_size),
        txt_add("06", rose_txt_style, 0, 0, canvas_size, canvas_size),
        txt_add("03", rose_txt_style, 0, 0, canvas_size, canvas_size)
    }

    result = {
        txt_radius_delta = 35,
        txt = rose_txt,
        canvas_size = canvas_size,
        canvas = rose_canvas,
        group = group_add(rose_canvas, rose_txt[1], rose_txt[2], rose_txt[3], rose_txt[4], rose_txt[5], rose_txt[6],
	                  rose_txt[7], rose_txt[8], rose_txt[9], rose_txt[10], rose_txt[11], rose_txt[12])
    }

    setmetatable(result, self)
    self.__index = self

    return result
end

local Compass = {
    draw = function (self)
        view = compass_views[enabled_compass_view]
        self.rose:draw()
        self.heading_bug:draw(heading, heading_bug_setting, view)
    end,

    visible = function (self, val)
        self.rose:visible(val)
        self.heading_bug:visible(val)
    end
}

function Compass:create(canvas_size)
    bug = Pointer:create(canvas_size, draw_heading_bug)
    r = Rose:create(canvas_size)

    result =  {
        rose = r,
        heading_bug = bug
    }

    setmetatable(result, self)
    self.__index = self

    return result
end

local function set_heading_bug(setting)
    heading_bug_setting = setting

    compass:draw()

    view = compass_views[enabled_compass_view]
    upper_display_update_heading_bug(heading_bug_setting)
    upper_display_park_heading_bug(helpers_park_pointer(heading, heading_bug_setting, view.viewable))

    compass:visible(true)
end

function compass_init(centre, size)

    compass_views = {
        View.create(centre, 176, 360),
        View.create({x = centre.x, y = 400}, 326, 70)
    }

    compass = Compass:create(size)

    compass:draw()
    compass:visible(false)
end

function compass_power_on(val)
    compass:visible(val)
end

function compass_update_heading(new_heading)
    heading = new_heading % 360
    upper_display_update_heading(heading)

    compass:draw()
end

function compass_update_heading_bug(direction)
    set_heading_bug((heading_bug_setting + direction) % 360)
end

function compass_reset_heading_bug()
    set_heading_bug(heading)
end

function compass_switch_view()
    enabled_compass_view = fif(enabled_compass_view < #compass_views, enabled_compass_view + 1, 1)
    compass:visible(true)
    view = compass_views[enabled_compass_view]
    compass:draw()
    upper_display_park_heading_bug(helpers_park_pointer(heading, heading_bug_setting, view.viewable))
end
