function helpers_x_position(start, length, deg)
    return start - math.sin(math.rad(deg)) * length
end

function helpers_y_position(start, length, deg)
    return start - math.cos(math.rad(deg)) * length
end

function helpers_get_object_location(origin, distance, bearing)
    return {
        x = helpers_x_position(origin.x, distance, bearing),
        y = helpers_y_position(origin.y, distance, bearing)
    }
end

-- rotate the centre of an object of size width x height by rotation degrees along an arc distance pixels from origin
function helpers_rotate_around_pt(node, origin, distance, rotation, width, height, keep_orientation)
    half_height = height / 2
    half_width = width / 2

    rotated_position = helpers_get_object_location(origin, distance, rotation)
    move(node, rotated_position.x - half_width, rotated_position.y - half_height, width, height)

    if not keep_orientation then
        rotate(node, -rotation)
    end
end

function helpers_format_heading(heading)
    return string.format("%03i°", math.floor(heading) % 360)
end

function helpers_format_distance(distance)
    return string.format("%.1fm", distance)
end

function helpers_format_speed(speed)
    return string.format("%i KT", math.floor(speed))
end

function helpers_format_name(prefix, index, tail)
    return string.format("%s%i%s", prefix, index, tail)
end

function helpers_park_pointer(heading1, heading2, viewable)
    delta = heading1 - heading2
    viewable = viewable / 2

    return math.abs(delta) > viewable and (360 - math.abs(delta)) > viewable
end

function helpers_visit_table(arr, func)
    for key, val in pairs(arr) do
        func(key, val)
    end
end

function helpers_visit_array(arr, func)
    for index, val in ipairs(arr) do
        func(index, val)
    end
end

function helpers_array_contains_value(arr, val)
    found = false
    helpers_visit_array(arr, function (index, value) found = found or val == value end)
    return found
end

-- see https://www.movable-type.co.uk/scripts/latlong.html
local TO_RAD = math.pi / 180
local TO_DEG = 180 / math.pi
local GT_CIRCLE_RADIUS = 21600 / (2 * math.pi)

function helpers_distance_and_bearing(pos1, pos2)
    lat1_rad = pos1.lat * TO_RAD
    lat2_rad = pos2.lat * TO_RAD
    lat_delta_rad = (pos2.lat - pos1.lat) * TO_RAD
    long_delta_rad = (pos2.long - pos1.long) * TO_RAD
    lat_half_delta_rad = lat_delta_rad / 2
    long_half_delta_rad = long_delta_rad / 2
    cos_lat1 = math.cos(lat1_rad)
    cos_lat2 = math.cos(lat2_rad)
    sq_half_chord = (math.sin(lat_half_delta_rad) * math.sin(lat_half_delta_rad)) +
        (cos_lat1 * cos_lat2 * math.sin(long_half_delta_rad) * math.sin(long_half_delta_rad))
    dist_rad = 2 * math.atan(math.sqrt(sq_half_chord), math.sqrt(1 - sq_half_chord))
    dist_naut = GT_CIRCLE_RADIUS * dist_rad

    sin_lat1 = math.sin(lat1_rad)
    sin_lat2 = math.sin(lat2_rad)
    y = math.sin(long_delta_rad) * cos_lat2
    x = (cos_lat1 * sin_lat2) -
        (sin_lat1 * cos_lat2 * math.cos(long_delta_rad))
    angle_rad = math.atan(y, x)
    bearing_deg = (angle_rad * TO_DEG + 360) % 360
    return {distance = dist_naut, bearing = bearing_deg}
end
