local bold_txt_style = "font:inconsolata_bold.ttf; size:24; valign:center; halign:center;"
local regular_txt_style = "font:inconsolata_regular.ttf; size:24; valign:center; halign:center; color:white;"
local regular_txt_right_style = "font:inconsolata_regular.ttf; size:24; valign:center; halign:right; color:white;"
local regular_txt_left_style = "font:inconsolata_regular.ttf; size:24; valign:center; halign:left; color:white;"

local source_txt_styles = nil
local back_course_txt = nil
local bearing_txts = nil
local avionics_power = false

local BearingTxt = {
    visible = function (self, val)
        visible(self.group, val and avionics_power)
    end
}

function BearingTxt:create(align, canvas_size)
    xpos = fif(align == "left", 100, canvas_size - 140)
    dist_xpos = fif(align == "left", xpos - 20, xpos - 5)
    label_ypos = canvas_size - 82

    label = txt_add("", regular_txt_left_style, xpos, label_ypos, 50, 30)
    dist = txt_add("---m", regular_txt_right_style, dist_xpos, label_ypos + 17, 65, 30)
    bearing = txt_add("---°", regular_txt_left_style, xpos, label_ypos + 37, 60, 30)

    result = {
        label_txt = label,
        dist_txt = dist,
        bearing_txt = bearing,
        group = group_add(label, dist, bearing)
    }

    setmetatable(result, self)
    self.__index = self

    return result
end

function lower_display_init(canvas_size)
    source_txt_styles = {}
    source_txt_styles["NAV1"] = "color:green"
    source_txt_styles["NAV2"] = "color:yellow"
    source_txt_styles["GPS"] = "color:cyan"

    back_course_txt = txt_add("BC", bold_txt_style, (canvas_size / 2) - 15, canvas_size - 95, 30, 30)

    bearing_txts = {
        BearingTxt:create("left", canvas_size),
        BearingTxt:create("right", canvas_size)
    }

    helpers_visit_array(bearing_txts, function (i, bt) BearingTxt.visible(bt, false) end)

    visible(back_course_txt, false)
end

function lower_display_update_bearing_data(bearing_index, label, distance, distance_valid, bearing, bearing_valid)
    bt = bearing_txts[bearing_index]

    txt_set(bt.label_txt, label)
    txt_set(bt.dist_txt, fif(distance_valid, helpers_format_distance(distance), "---m"))
    txt_set(bt.bearing_txt, fif(bearing_valid, helpers_format_heading(bearing), "---°"))

    active_style = source_txt_styles[label]
    txt_style(bt.label_txt, active_style)
    txt_style(bt.dist_txt, active_style)
    txt_style(bt.bearing_txt, active_style)
end

function lower_display_bearing_visible(bearing_index, val)
    bearing_txts[bearing_index]:visible(val)
end

function lower_display_back_course_visible(nav_source_index, val)
    txt_style(back_course_txt, source_txt_styles[string.format("NAV%i", nav_source_index)])
    visible(back_course_txt, val and avionics_power)
end

function lower_display_power_on(val)
    avionics_power = val
    helpers_visit_array(bearing_txts, function (i, bt) bt:visible(val) end)
end
