local selected_source_index = 1
local sources = nil
local selected_view_index = 1
local current_heading = 0
local avionics_power = false
local map_enabled = false
local course_pointer_height = 43
local course_pointer_width = 30
local views = nil
local source_colours = nil
local nav_view = nil

local function draw_course_head(view)
    centre = view.centre
    radius = view.size + 3
    cross_height = centre.y - radius + 17
    source = sources[selected_source_index]
    half_width = course_pointer_width / 2

    _move_to(centre.x, centre.y - radius)
    _line_to(centre.x, centre.y - radius + course_pointer_height)
    _move_to(centre.x - half_width, cross_height)
    _line_to(centre.x + half_width, cross_height)
    _stroke(source_colours[selected_source_index], 6)

    if source.to_from == 1 then
        _triangle(centre.x - half_width - 10, centre.y - radius + course_pointer_height,
                centre.x - half_width + 10, centre.y - radius + course_pointer_height,
                centre.x - half_width, centre.y - radius + course_pointer_height - 20)
        _fill("white")
    elseif source.to_from == 2 then
        _triangle(centre.x - half_width - 10, centre.y - radius + course_pointer_height - 20,
                centre.x - half_width + 10, centre.y - radius + course_pointer_height - 20,
                centre.x - half_width, centre.y - radius + course_pointer_height)
        _fill("white")
    end
end

local function draw_course_tail(view)
    centre = view.centre
    radius = view.size + 3

    _move_to(centre.x, centre.y + radius)
    _line_to(centre.x, centre.y + radius - course_pointer_height)
    _stroke(source_colours[selected_source_index], 6)
end

local function source_has_valid_vdi(source)
    return source.has_vdi and not source.has_vdi_flag
end


local function get_scaled_c_deflection(deflection)
    return fif(deflection == 0, 0, 90 * deflection / 127)
end

local function get_scaled_v_deflection(deflection)
    return fif(deflection == 0, 0, 90 * deflection / 119)
end

local InnerCdi = {
    draw = function (self, obs, deflection, centre)
        scaled_deflection = get_scaled_c_deflection(deflection)
        rotation = obs - current_heading
        rotate(self.bg_img, rotation)
        canvas_draw(self.canvas, function ()
            _rotate(rotation)
            _move_to(centre.x + scaled_deflection, centre.y - 110)
            _line_to(centre.x + scaled_deflection, centre.y + 110)
            _stroke(source_colours[selected_source_index], 6)
            _rotate(-rotation)
        end)
    end,

    visible = function (self, viewable, val)
        visible(self.group, viewable == 360 and val)
    end
}

function InnerCdi:create(c, canvas_size)
    bg = img_add("cdi_vdi_bg.png", c.x - 95, c.y - 7, 190, 14)
    canvas = canvas_add(0, 0, canvas_size, canvas_size)

    result = {
        bg_img = bg,
        canvas = canvas,
        group = group_add(bg, canvas)
    }

    setmetatable(result, self)
    self.__index = self

    return result
end

local OuterCdi = {
    draw = function (self, source)
        scaled_deflection = get_scaled_c_deflection(fif(source.back_course_on, -source.c_deflection,source.c_deflection))
        source = sources[selected_source_index]

        canvas_draw(self.canvas, function ()
            if source.has_nav then
                _move_to(95 + scaled_deflection, 0)
                _line_to(95 + scaled_deflection, 40)
                _stroke(source_colours[selected_source_index], 6)
            else
                _move_to(0, 0)
                _line_to(190, 40)
                _move_to(190, 0)
                _line_to(0, 40)
                _stroke("red", 6)
            end
        end)
    end,

    visible = function (self, val)
        visible(self.group, val)
    end
}

function OuterCdi:create(c, canvas_size)
    bg = img_add("cdi_vdi_bg.png", c.x - 95, canvas_size - 59, 190, 14)
    canvas = canvas_add(c.x - 95, canvas_size - 72, 190, 40)

    result = {
        bg_img = bg,
        canvas = canvas,
        group = group_add(bg, canvas)
    }

    setmetatable(result, self)
    self.__index = self

    return result
end

local InnerVdi = {
    draw = function (self, obs, deflection, centre)
        scaled_deflection = get_scaled_v_deflection(deflection)
        rotation = obs - current_heading
        canvas_draw(self.canvas, function ()
            _rotate(rotation)
            _move_to(centre.x - 110, centre.y + scaled_deflection)
            _line_to(centre.x + 110, centre.y + scaled_deflection)
            _stroke(source_colours[selected_source_index], 6)
            _rotate(-rotation)
        end)
    end,

    visible = function (self, viewable, val)
        visible(self.group, val and viewable == 360)
    end
}

function InnerVdi:create(c, canvas_size)
    canvas = canvas_add(0, 0, canvas_size, canvas_size)

    result = {
        canvas = canvas,
        group = group_add(canvas)
    }

    setmetatable(result, self)
    self.__index = self

    return result
end

local OuterVdi = {
    draw = function (self, deflection,centre)
        canvas_size = 2 * centre.x
        scaled_deflection = get_scaled_v_deflection(deflection)
        source = sources[selected_source_index]
        flag = source.has_vdi_flag
        warning = source_has_valid_vdi(source) and source.back_course_on

        canvas_draw(self.canvas, function ()
            if flag or warning then
                _move_to(0, 0)
                _line_to(40, 190)
                _move_to(0, 190)
                _line_to(40, 0)
                _stroke(fif(flag, "red", "yellow"), 6)
            else
                _move_to(0, 95 + scaled_deflection)
                _line_to(40, 95 + scaled_deflection)
                _stroke(source_colours[selected_source_index], 6)
            end
        end)
        rotate(self.bg_img, 90)
    end,

    visible = function (self, val)
        visible(self.group, val)
    end
}

function OuterVdi:create(c, canvas_size)
    bg = img_add("cdi_vdi_bg.png", canvas_size - 148, c.y - 7, 190, 14)
    canvas = canvas_add(canvas_size - 72, c.y - 95, 40, 190)

    result = {
        bg_img = bg,
        canvas = canvas,
        group = group_add(bg, canvas)
    }

    setmetatable(result, self)
    self.__index = self

    return result
end

local NavView = {
    draw = function (self)
        view = views[selected_view_index]
        source = sources[selected_source_index]
        centre = view.centre

        move(self.airplane_img, centre.x - 15, centre.y - 15, 30, 30)

        self.crse_ptr:draw(current_heading, source.obs, view)
        self.inner_cdi:draw(source.obs, source.c_deflection, view.centre)
        self.inner_vdi:draw(source.obs, source.v_deflection, view.centre)
        self.outer_cdi:draw(source)
        self.outer_vdi:draw(source.v_deflection,view.centre)

        display_name = fif(source.has_vdi, "ILS", fif(source.has_loc, "LOC", "NAV"))

        if (source.has_nav) then
            upper_display_update_course(display_name, selected_source_index, source.obs)
            upper_display_park_course_ptr(helpers_park_pointer(current_heading, source.obs, view.viewable))
        else
            upper_display_course_unavailable(display_name, selected_source_index)
        end

        if source.has_dme then
            upper_display_update_nav_data(selected_source_index, source.dme_dist, source.dme_spd)
        else
            upper_display_nav_data_unavailable(selected_source_index)
        end

        lower_display_back_course_visible(selected_source_index, source.back_course_on)
    end,

    visible = function (self, value)
        view = views[selected_view_index]
        source = sources[selected_source_index]
        inner_allowed = view.viewable == 360 and not map_enabled
        inner_vdi_allowed = inner_allowed and source_has_valid_vdi(source) and not source.back_course_on
        outer_vdi_allowed = source_has_valid_vdi(source) or source.has_vdi_flag

        self.crse_ptr:visible(value and source.has_nav)
        self.inner_cdi:visible(view.viewable, inner_allowed and source.has_nav and value)
        self.inner_vdi:visible(view.viewable, inner_vdi_allowed and value)
        visible(self.airplane_img, value and not inner_allowed)
        self.outer_cdi:visible(value)
        self.outer_vdi:visible(outer_vdi_allowed and value)
    end
}

function NavView:create(centre, canvas_size)
    result =  {
        crse_ptr = Pointer:create(canvas_size, draw_course_head, draw_course_tail),
        inner_cdi = InnerCdi:create(centre, canvas_size),
        inner_vdi = InnerVdi:create(centre, canvas_size),
        airplane_img = img_add_fullscreen("airplane.png"),
        outer_cdi = OuterCdi:create(centre, canvas_size),
        outer_vdi = OuterVdi:create(centre, canvas_size)
    }

    setmetatable(result, self)
    self.__index = self

    return result
end

local Source = {
    update_data = function (self, obs, has_nav, has_loc, has_vdi, has_vdi_flag, c_def, v_def, to_from, has_dme, dme_spd, dme_dist)
        heading_diff = math.abs(current_heading - obs)
        self.obs = obs
        self.back_course_on = has_nav and fif(heading_diff > 180, 360 - heading_diff, heading_diff) > 90
        self.has_nav = has_nav
        self.has_loc = has_loc
        self.has_vdi_flag = has_vdi_flag
        self.has_vdi = has_nav and has_vdi
        self.c_deflection = c_def
        self.v_deflection = v_def
        self.to_from = to_from
        self.has_dme = has_dme and has_nav
        self.dme_spd = dme_spd
        self.dme_dist = dme_dist
    end
}

function Source:create()
    result = {
        obs = 0,
        c_deflection = 0,
        v_deflection = 0,
        has_dme = false,
        has_nav = false,
        has_vdi = false,
        has_vdi_flag = false,
        back_course_on = false,
        to_from = 0,
        dme_spd = 0,
        dme_dist = 0,
    }

    setmetatable(result, self)
    self.__index = self

    return result
end

function nav_init(centre, canvas_size)
    sources = {
        Source:create(),
        Source:create()
    }

    source_colours = {
        "green",
        "yellow"
    }

    views = {
        View.create(centre, 152, 360, canvas_size),
        View.create({x = centre.x, y = 400}, 302, 70, canvas_size)
    }

    nav_view = NavView:create(centre, canvas_size)

    nav_view:draw()
    nav_view:visible(false)
end

function nav_power_on(val)
    avionics_power = val
    nav_view:visible(val)
end

function nav_update_data(heading,
                         obs1, has_nav1, has_loc1, has_gs1, gs_flag1, c_def1, v_def1, to_from1, has_dme1, dme_dist1, dme_spd1,
                         obs2, has_nav2, has_loc2, has_gs2, gs_flag2, c_def2, v_def2, to_from2, has_dme2, dme_dist2, dme_spd2)
    current_heading = heading
    sources[1]:update_data(obs1, has_nav1, has_loc1, has_gs1, gs_flag1, c_def1, v_def1, to_from1, has_dme1, dme_spd1, dme_dist1)
    sources[2]:update_data(obs2, has_nav2, has_loc2, has_gs2, gs_flag2, c_def2, v_def2, to_from2,has_dme2, dme_spd2, dme_dist2)

    nav_view:visible(avionics_power)
    nav_view:draw()
end

function nav_switch_view()
    selected_view_index = fif(selected_view_index < #views, selected_view_index + 1, 1)
    selected_view = views[selected_view_index]
    selected_source = sources[selected_source_index]

    nav_view:visible(avionics_power)
    bearings_set_view(selected_view.viewable)
    nav_view:draw()
end

function nav_switch_source()
    selected_source_index = fif(selected_source_index < #sources, selected_source_index + 1, 1)
    upper_display_switch_nav_source(selected_source_index)

    nav_view:draw()
end

function nav_update_obs(direction)
    fsx_event(string.format("VOR%i_OBI_%s", selected_source_index, fif(direction == 1, "INC", "DEC")))
end

function nav_set_map_enabled(val)
    map_enabled = val
    nav_view:draw()
end
