local MAP_STATUS_AIRP = "AIRPORT"
local MAP_STATUS_VOR = "VHF_NAVAID"
local MAP_STATUS_NDB = "NDB"
local MAP_STATUS_WAYPOINT = "WAYPOINT"
local MAP_STATUS_RESTRICTED = "RESTRICTED_"
local bold_txt_style = "font:inconsolata_bold.ttf; size:25; valign:center; halign:center; color:white;"
local regular_txt_style = "font:inconsolata_regular.ttf; size:25; valign:center; halign:center; color:white;"
local regular_txt_right_style = "font:inconsolata_regular.ttf; size:25; valign:center; halign:right; color:white;"
local regular_txt_left_style = "font:inconsolata_regular.ttf; size:25; valign:center; halign:left; color:white;"
local map_txt_right_style = "font:inconsolata_regular.ttf; size:25; valign:center; halign:right; color:magenta;"
local map_txt_anun_style = "font:inconsolata_regular.ttf; size:25; valign:center; halign:center; color:black; background_color:cyan;"
local source_txt_styles = nil
local source_colours = nil
local nav_source_name = "NAV"
local nav_selected_source = 1
local bearing1_selected_source = 1
local bearing2_selected_source = 2

local avionics_power = false

local parked_canvas = nil
local heading_bug_parked = false
local course_ptr_parked = false
local bearing_ptrs_parked = nil

local heading_txt = nil
local heading_bug_txt = nil

local course_bearing_txt = nil
local course_label_txt = nil
local course_dist_txt = nil
local course_gs_txt = nil

local map_rng_txt = nil
local map_anun_txt = nil
local map_group = nil


local txt_group = nil

local annunciation_imgs = nil
local annunciation_group = nil

function update_annunciation(nav_source_name, nav_source_index)
    visible(annunciation_group, false)

    nav_selected_source = nav_source_index
    nav_source_name = nav_source_name
    visible(annunciation_imgs[nav_source_index][nav_source_name], avionics_power)
end

local function draw_heading_bug(position)
    height = 10
    half_height = height / 2
    half_width = 15
    top = position.y - half_height
    bottom = top + height
    rect_width = 9
    triangle_width = half_width - rect_width

    _rotate(45, position.x, position.y)
    _rect(position.x - half_width, top, rect_width, height)
    _fill("white")
    _rect(position.x + triangle_width, top, rect_width, height)
    _fill("white")
    _triangle(position.x - triangle_width, top, position.x - triangle_width, bottom, position.x, bottom)
    _fill("white")
    _triangle(position.x + triangle_width, top, position.x + triangle_width, bottom, position.x, bottom)
    _fill("white")
    _rotate(-45, position.x, position.y)
end

local function draw_bearing1(position)
    _rotate(45, position.x, position.y)
    for i = position.y - 2, position.y + 2 do
        _move_to(position.x, i)
        _line_to(position.x - 15, i + 10)
        _move_to(position.x, i)
        _line_to(position.x + 15, i + 10)
        _stroke(source_colours[bearing1_selected_source], 1.5)
    end
    _rotate(-45, position.x, position.y)
end

local function draw_bearing2(position)
    _rotate(45, position.x, position.y)
    for i = position.y - 2, position.y + 2 do
        _move_to(position.x - 1.5, i)
        _line_to(position.x - 15, i + 9)
        _move_to(position.x + 1.5, i)
        _line_to(position.x + 15, i + 9)
        _stroke(source_colours[bearing2_selected_source], 1.5)
    end
    _rotate(-45, position.x, position.y)
end

local function draw_course_ptr(position)
    half_height = 11
    cross_height = 4
    half_width = 7

    _rotate(45, position.x, position.y)
    _move_to(position.x, position.y - half_height)
    _line_to(position.x, position.y + half_height)
    _move_to(position.x - half_width, position.y - cross_height)
    _line_to(position.x + half_width, position.y - cross_height)
    _stroke(source_colours[nav_selected_source], 3)
    _rotate(-45, position.x, position.y)
end

local function draw_parked()
    if heading_bug_parked then
        draw_heading_bug({x = 50, y = 20})
    end

    if bearing_ptrs_parked[1] and avionics_power then
        draw_bearing1({x = 43, y = 27})
    end

    if bearing_ptrs_parked[2] and avionics_power then
        draw_bearing2({x = 35, y = 35})
    end

    if course_ptr_parked and avionics_power then
        draw_course_ptr({x = 23, y = 47})
    end

end

function upper_display_init(canvas_size)
    source_txt_styles = {
        "color:green",
        "color:yellow"
    }

    source_colours = {
        "green",
        "yellow"
    }

    heading_txt = txt_add(helpers_format_heading(0), bold_txt_style, (canvas_size / 2) - 30, 15, 60, 26)
    heading_bug_txt = txt_add(helpers_format_heading(0), bold_txt_style, 380, 52, 60, 26)

    course_bearing_txt = txt_add(helpers_format_heading(0), bold_txt_style, 60, 52, 60, 26)
    course_dist_txt = txt_add(helpers_format_distance(0), regular_txt_right_style, 57, 32, 90, 26)
    course_gs_txt = txt_add(helpers_format_speed("0"), regular_txt_right_style, 350, 15, 90, 26)
    course_label_txt = txt_add("", regular_txt_left_style, 60, 15, 130, 26)

    map_rng_txt = txt_add(string.format("RNG 40m"), map_txt_right_style, 355, 32, 100, 26)
    map_anun_txt = txt_add("", map_txt_anun_style, 32, 140, 20, 20)
    map_group = group_add(map_rng_txt, map_anun_txt)

    map_status_imgs = {}
    map_status_imgs[MAP_STATUS_WAYPOINT] = img_add("status_gps.png", 32, 160, 20, 20)
    map_status_imgs[MAP_STATUS_AIRP] = img_add("airport.png", 32, 180, 20, 20)
    map_status_imgs[MAP_STATUS_VOR] = img_add("vor.png", 32, 200, 20, 20)
    map_status_imgs[MAP_STATUS_NDB] = img_add("ndb.png", 52, 160, 20, 20)
    map_status_imgs[MAP_STATUS_RESTRICTED .. MAP_STATUS_WAYPOINT] = img_add("restricted_gps.png", 32, 160, 20, 20)
    map_status_imgs[MAP_STATUS_RESTRICTED .. MAP_STATUS_AIRP] = img_add("restricted_airport.png", 32, 180, 20, 20)
    map_status_imgs[MAP_STATUS_RESTRICTED .. MAP_STATUS_VOR] = img_add("restricted_vor.png", 32, 200, 20, 20)
    map_status_imgs[MAP_STATUS_RESTRICTED .. MAP_STATUS_NDB] = img_add("restricted_ndb.png", 52, 160, 20, 20)

    parked_canvas = canvas_add(canvas_size - 90, 50, 100, 100)
    bearing_ptrs_parked = {false, false}

    txt_group = group_add(heading_txt, heading_bug_txt,
                          course_bearing_txt, course_dist_txt, course_gs_txt, course_label_txt)

    annunciation_imgs = {}
    for index = 1,2 do
        annunciation_imgs[index] = {}
        annunciation_imgs[index]["NAV"] = img_add(helpers_format_name("nav", index, "_anun.png"), 30, 60, 21, 78)
        annunciation_imgs[index]["LOC"] = img_add(helpers_format_name("loc", index, "_anun.png"), 30, 60, 21, 78)
        annunciation_imgs[index]["ILS"] = img_add(helpers_format_name("ils", index, "_anun.png"), 30, 60, 21, 78)
    end

    for index, index_annunciations in ipairs(annunciation_imgs) do
        for source, img in pairs(index_annunciations) do
            if annunciation_group then
                group_obj_add(annunciation_group, img)
            else
                annunciation_group = group_add(img)
            end
        end
    end

    helpers_visit_table(map_status_imgs, function (src, img) visible(img, false) end)

    visible(txt_group, false)

    visible(map_group, false)
    visible(annunciation_group, false)
end

function upper_display_switch_nav_source(index)
    nav_selected_source = index
    canvas_draw(parked_canvas, draw_parked)
end

function upper_display_park_bearing_ptr(index, val)
    bearing_ptrs_parked[index] = val
    canvas_draw(parked_canvas, draw_parked)
end

function upper_display_park_heading_bug(val)
    heading_bug_parked = val
    canvas_draw(parked_canvas, draw_parked)
end

function upper_display_update_heading_bug(bearing)
    txt_set(heading_bug_txt, helpers_format_heading(bearing))
end

function upper_display_update_heading(bearing)
    txt_set(heading_txt, helpers_format_heading(bearing))
end

function upper_display_park_course_ptr(val)
    course_ptr_parked = val
    canvas_draw(parked_canvas, draw_parked)
end

function upper_display_update_course(nav_source_name, nav_source_index, bearing)
    formatted_bearing = helpers_format_heading(bearing)
    txt_set(course_label_txt, string.format("%s%i %s", nav_source_name, nav_source_index, formatted_bearing))
    txt_style(course_label_txt, source_txt_styles[nav_source_index])

    txt_set(course_bearing_txt, formatted_bearing)
    txt_style(course_bearing_txt, source_txt_styles[nav_source_index])

    update_annunciation(nav_source_name, nav_source_index)
end

function upper_display_course_unavailable(nav_source_name, nav_source_index)
    txt_set(course_label_txt, string.format("%s%i ---°", nav_source_name, nav_source_index))
    txt_style(course_label_txt, source_txt_styles[nav_source_index])

    txt_set(course_bearing_txt, "---°")
    txt_style(course_bearing_txt, source_txt_styles[nav_source_index])

    update_annunciation(nav_source_name, nav_source_index)
end

function upper_display_update_nav_data(nav_source_index, dist, gs)
    txt_set(course_dist_txt, helpers_format_distance(dist))
    txt_style(course_dist_txt, source_txt_styles[nav_source_index])

    txt_set(course_gs_txt, helpers_format_speed(gs))
    txt_style(course_gs_txt, source_txt_styles[nav_source_index])
end

function upper_display_nav_data_unavailable(nav_source_index)
    txt_set(course_dist_txt, "---m")
    txt_style(course_dist_txt, source_txt_styles[nav_source_index])

    txt_set(course_gs_txt, "--- KT")
    txt_style(course_gs_txt, source_txt_styles[nav_source_index])
end

function upper_display_update_map_data(map_enabled, selected_memory, selected_layers, range)
    txt_set(map_anun_txt, string.format("%d", selected_memory))
    txt_set(map_rng_txt, string.format("RNG %dm", range))
    visible(map_group, map_enabled)

    helpers_visit_table(map_status_imgs,
        function (key, img)
            visible(img, map_enabled and helpers_array_contains_value(selected_layers, key))
        end)
end

function upper_display_power_on(val)
    avionics_power = val

    canvas_draw(parked_canvas, draw_parked)

    visible(txt_group, avionics_power)
    visible(annunciation_imgs[nav_selected_source][nav_source_name], avionics_power)

    if not val then
        visible(map_group, val)
        helpers_visit_table(map_status_imgs, function (src, img) visible(img, false) end)
    end
end
