local centre = {x = 250, y = 250}
local size = 500

local avionics_power = false

img_add_fullscreen("screen_background.png")
upper_display_init(size)
lower_display_init(size)
map_init(centre, size)
compass_init(centre, size)
bearings_init(centre, size)
nav_init(centre, size)
img_add_fullscreen("bezel.png")

local function view_button_callback()
    if avionics_power then
        compass_switch_view()
        nav_switch_view()
        map_switch_view()
    end
end

local function nav_button_callback()
    if avionics_power then
        nav_switch_source()
    end
end

local function obs_update_callback(direction)
    if avionics_power then
        nav_update_obs(direction)
    end
end

local function bearing_button_callback()
    if avionics_power then
        bearings_select()
    end
end

local function zoom_in_button_callback()
    if avionics_power then
        map_zoom_in()
    end
end

local function zoom_out_button_callback()
    if avionics_power then
        map_zoom_out()
    end
end

local function map_button_callback()
    if avionics_power then
        map_enabled = map_switch_memory()
        nav_set_map_enabled(map_enabled)
    end
end

local function clear_button_callback()
    if avionics_power then
        map_toggle_clear()
    end
end

dial_add("bug_knob.png", 440, 406, 60, 60, compass_update_heading_bug)
dial_add("crse_knob.png", 0, 406, 60, 60, obs_update_callback)
button_add("view_button.png", "view_button_pressed.png", 469, 96, 30, 65, view_button_callback)
button_add("nav_button.png", "nav_button_pressed.png", 1, 70, 30, 65, nav_button_callback)
button_add("bearing_button.png", "bearing_button_pressed.png", 230, 469, 65, 30, bearing_button_callback)
button_add("clear_button.png", "clear_button_pressed.png", 140, 469, 65, 30, clear_button_callback)
button_add("zoom_in_button.png", "zoom_in_button_pressed.png", 469, 270, 30, 65, zoom_in_button_callback)
button_add("zoom_out_button.png", "zoom_out_button_pressed.png", 469, 200, 30, 65, zoom_out_button_callback)
button_add("map_button.png", "map_button_pressed.png", 1, 150, 30, 65, map_button_callback)

local function power_callback(avionics_switch, avionics_volts)
    avionics_power = avionics_switch and avionics_volts > 10

    compass_power_on(avionics_power)
    nav_power_on(avionics_power)
    upper_display_power_on(avionics_power)
    lower_display_power_on(avionics_power)
    bearings_power_on(avionics_power)
    map_power_on(avionics_power)
end

local function location_callback(lat, long, true_heading)
    map_update_location(lat, long, true_heading)
end

local function gps_data_callback(gps_wp_active, gps_direct_to,
                                 next_wp_id, next_wp_lat, next_wp_long,
                                 prev_wp_valid,
                                 prev_wp_id, prev_wp_lat, prev_wp_long)
    map_update_gps_data(gps_wp_active, gps_direct_to,
                        next_wp_id, next_wp_lat, next_wp_long,
                        prev_wp_valid,
                        prev_wp_id, prev_wp_lat, prev_wp_long)
end

local function nav_data_callback(heading,
                                 obs1, has_nav1, has_loc1, has_gs1, c_def1, v_def1, to_from1, rel_bearing1, has_dme1, dme_dist1, dme_spd1,
                                 obs2, has_nav2, has_loc2, has_gs2, c_def2, v_def2, to_from2, rel_bearing2, has_dme2, dme_dist2, dme_spd2)
    gs_flag1 = has_loc1 and not has_gs1
    gs_flag2 = has_loc2 and not has_gs2
    heading = heading or 0

    compass_update_heading(heading)
    nav_update_data(heading or 0,
                    obs1 or 0, has_nav1, has_loc1, has_gs1, gs_flag1, c_def1, v_def1, to_from1, has_dme1, dme_dist1, dme_spd1,
                    obs2 or 0, has_nav2, has_loc2, has_gs2, gs_flag2, c_def2, v_def2, to_from2, has_dme2, dme_dist2, dme_spd2)
    bearings_update_source_data(heading,
                                has_nav1, rel_bearing1 or 0, has_dme1, dme_dist1,
                                has_nav2, rel_bearing2 or 0, has_dme2, dme_dist2)
end

fsx_variable_subscribe("AVIONICS MASTER SWITCH", "Bool",
                       "ELECTRICAL AVIONICS BUS VOLTAGE", "Volts",
                       power_callback)

fsx_variable_subscribe("PLANE LATITUDE", "Degrees",
                       "PLANE LONGITUDE", "Degrees",
                       "PLANE HEADING DEGREES TRUE", "Degrees",
                       location_callback)

fsx_variable_subscribe("GPS IS ACTIVE WAY POINT", "Bool",
                       "GPS IS DIRECTTO FLIGHTPLAN", "Bool",
                       "GPS WP NEXT ID", "String",
                       "GPS WP NEXT LAT", "Degrees",
                       "GPS WP NEXT LON", "Degrees",
                       "GPS WP PREV VALID", "Bool",
                       "GPS WP PREV ID", "String",
                       "GPS WP PREV LAT", "Degrees",
                       "GPS WP PREV LON", "Degrees",
                       gps_data_callback)

fsx_variable_subscribe("HEADING INDICATOR", "Degrees",
                       "NAV OBS:1", "Degrees",
                       "NAV HAS NAV:1", "Bool",
                       "NAV HAS LOCALIZER:1", "Bool",
                       "NAV HAS GLIDE SLOPE:1", "Bool",
                       "NAV CDI:1", "Number",
                       "NAV GSI:1", "Number",
                       "NAV TOFROM:1", "Enum",
                       "NAV RELATIVE BEARING TO STATION:1", "Degrees",
	                   "NAV HAS DME:1", "Bool",
		               "NAV DME:1", "NAUTICAL MILES",
		               "NAV DMESPEED:1", "Knots",
                       "NAV OBS:2", "Degrees",
                       "NAV HAS NAV:2", "Bool",
                       "NAV HAS LOCALIZER:2", "Bool",
                       "NAV HAS GLIDE SLOPE:2", "Bool",
                       "NAV CDI:2", "Number",
                       "NAV GSI:2", "Number",
                       "NAV TOFROM:2", "Enum",
                       "NAV RELATIVE BEARING TO STATION:2", "Degrees",
	                   "NAV HAS DME:2", "Bool",
		               "NAV DME:2", "NAUTICAL MILES",
		               "NAV DMESPEED:2", "Knots",
                       nav_data_callback)
